
class Organizer(object):

    def __init__(self, id, email, phone, twitter, info):
        self.id = id
        self.email = email
        self.phone = phone
        self.twitter = twitter
        self.info = info

    @staticmethod
    def parse(data):
        return Organizer(
            data['ownerId'],
            data['contactEmail'],
            data['contactPhone'],
            data['contactTwitter'],
            data['contactInfo']
        )

    # GETTERS
    def get_id(self):
        return self.id

    def get_email(self):
        return self.email

    def get_phone(self):
        return self.phone

    def get_twitter(self):
        return self.twitter

    def get_info(self):
        return self.info




class Attendee(object):

    def __init__(self, id, gamer_tag, prefix, created_at, claimed,
                 verified, player_id, phone_number, connected_accounts,
                 contact_info, event_ids):
        self.id = id
        self.gamer_tag = gamer_tag
        self.prefix = prefix
        self.created_at = created_at
        self.claimed = claimed
        self.verified = verified
        self.player_id = player_id
        self.phone_number = phone_number
        self.connected_accounts = connected_accounts
        self.contact_info = contact_info
        self.event_ids = event_ids

    @staticmethod
    def parse(data):
        return Attendee(
            data['id'],
            data['gamerTag'],
            data['prefix'],
            data['createdAt'],
            data['claimed'],
            data['verified'],
            data['playerId'],
            data['phoneNumber'],
            data['connectedAccounts'],
            data['contactInfo'],
            [ids for ids in data['events']]
        )

    # GETTERS
    def get_id(self):
        return self.id
    
    def get_gamer_tag(self):
        return self.gamer_tag
    
    def get_prefix(self):
        return self.prefix
    
    def get_created_at(self):
        return self.created_at
    
    def get_claimed(self):
        return self.claimed
    
    def get_verified(self):
        return self.verified
    
    def get_player_id(self):
        return self.player_id
    
    def get_phone_number(self):
        return self.phone_number
    
    def get_connected_accounts(self):
        return self.connected_accounts
    
    def get_contact_info(self):
        return self.contact_info
    
    def get_event_ids(self):
        return self.event_ids
    



class StreamQueue(object):

    def __init__(self, stream, sets):
        self.stream = stream
        self.sets = sets

    @staticmethod
    def parse(data):
        return StreamQueue(

        )

    def get_stream(self):
        return self.stream

    def get_sets(self):
        return self.sets


import smashggpy.queries.Tournament_Queries as queries
from smashggpy.util.Logger import Logger
from smashggpy.common.Common import flatten
from smashggpy.util.NetworkInterface import NetworkInterface as NI
from smashggpy.models.Venue import Venue
from smashggpy.models.Organizer import Organizer

class Tournament(object):

    def __init__(self, id, name, slug, start_time, end_time, timezone, venue, organizer):
        self.id = id
        self.name = name
        self.slug = slug
        self.start_time = start_time
        self.end_time = end_time
        self.timezone = timezone
        self.venue = venue
        self.organizer = organizer

    @staticmethod
    def get(slug: str):
        data = NI.query(queries.get_tournament_by_slug, {'slug': slug})
        base_data = data['data']['tournament']
        return Tournament.parse(base_data)

    @staticmethod
    def get_by_id(id: int):
        data = NI.query(queries.get_tournament_by_id, {'id': id})
        base_data = data['data']['tournament']
        return Tournament.parse(base_data)

    @staticmethod
    def parse(data):
        return Tournament(
            data['id'],
            data['name'],
            data['slug'],
            data['startAt'],
            data['endAt'],
            data['timezone'],
            Venue.parse(data),
            Organizer.parse(data)
        )

    def get_events(self):
        data = NI.query(queries.get_tournament_events, {'id': self.id})
        base_data = data['data']['tournament']['events']
        return [Event.parse(event_data) for event_data in base_data]

    def get_phases(self):
        data = NI.query(queries.get_tournament_phases, {'id': self.id})
        base_events = data['data']['tournament']['events']
        phases = []
        for event in base_events:
            for event_phase in event['phases']:
                phases.append(Phase.parse(event_phase))
        return phases

    def get_phase_groups(self):
        data = NI.query(queries.get_tournament_phase_groups, {'id': self.id})
        base_events = data['data']['tournament']['events']
        phase_groups = []
        for event in base_events:
            for event_phase_groups in event['phaseGroups']:
                phase_groups.append(PhaseGroup.parse(event_phase_groups))
        return phase_groups

    def get_attendees(self):
        Logger.info('Getting Attendees for Tournament: {0}:{1}'.format(self.id, self.name))
        phase_groups = self.get_phase_groups()
        attendees = flatten([phase_group.get_attendees() for phase_group in phase_groups])
        return attendees

    def get_entrants(self):
        Logger.info('Getting Entrants for Tournament: {0}:{1}'.format(self.id, self.name))
        Logger.warning('Aggregate queries ')
        phase_groups = self.get_phase_groups()
        entrants = flatten([phase_group.get_entrants() for phase_group in phase_groups])
        return entrants

    def get_sets(self):
        Logger.info('Getting Sets for Tournament: {0}:{1}'.format(self.id, self.name))
        phase_groups = self.get_phase_groups()
        sets = flatten([phase_group.get_sets() for phase_group in phase_groups])
        return sets

    def get_incomplete_sets(self):
        Logger.info('Getting Incomplete Sets for Tournament: {0}:{1}'.format(self.id, self.name))
        sets = self.get_sets()
        incomplete_sets = []
        for ggset in sets:
            if ggset.get_is_completed() is False:
                incomplete_sets.append(ggset)
        return incomplete_sets

    def get_completed_sets(self):
        Logger.info('Getting Completed Sets for Tournament: {0}:{1}'.format(self.id, self.name))
        sets = self.get_sets()
        complete_sets = []
        for ggset in sets:
            if ggset.get_is_completed() is True:
                complete_sets.append(ggset)
        return complete_sets

    # GETTERS
    def get_id(self):
        return self.id

    def get_name(self):
        return self.name

    def get_slug(self):
        return self.slug

    def get_start_time(self):
        return self.start_time

    def get_end_time(self):
        return self.end_time

    def get_timezone(self):
        return self.timezone

    def get_venue(self):
        return self.venue

    def get_organizer(self):
        return self.organizer


from smashggpy.models.Event import Event
from smashggpy.models.Phase import Phase
from smashggpy.models.PhaseGroup import PhaseGroup



class NotInitializedException(Exception):
	def __init__(self, component_name):
		self.message = '{} has not been initialized!'.format(component_name)

class BadTokenException(Exception): 
	def __init__(self, token='No token handed to Exception'):
		self.message = 'Bad Token. Please input a 32 character hexidecimal string.\nGot input "{}"'.format(token)


import smashggpy.queries.Phase_Group_Queries as queries
from smashggpy.common.Common import flatten
from smashggpy.util.Logger import Logger
from smashggpy.util.NetworkInterface import NetworkInterface as NI

class PhaseGroup(object):

    def __init__(self, id, display_identifier, first_round_time, state, phase_id, wave_id, tiebreak_order):
        self.id = id
        self.display_identifier = display_identifier
        self.first_round_time = first_round_time
        self.state = state
        self.phase_id = phase_id
        self.wave_id = wave_id
        self.tiebreak_order = tiebreak_order

    @staticmethod
    def get(id: int):
        data = NI.query(queries.phase_group_by_id, {'id': id})
        base_data = data['data']['phaseGroup']
        return PhaseGroup.parse(base_data)

    @staticmethod
    def parse(data):
        return PhaseGroup(
            data['id'],
            data['displayIdentifier'],
            data['firstRoundTime'],
            data['state'],
            data['phaseId'],
            data['waveId'],
            data['tiebreakOrder']
        )

    def get_attendees(self):
        Logger.info('Getting Attendees for phase group: {0}:{1}'.format(self.id, self.display_identifier))
        data = NI.paginated_query(queries.phase_group_attendees, {'id': self.id})
        participants = flatten([entrant_data['entrant']['participants'] for entrant_data in data])
        attendees = [Attendee.parse(participant_data) for participant_data in participants]
        return attendees

    def get_entrants(self):
        Logger.info('Getting Entrants for phase group: {0}:{1}'.format(self.id, self.display_identifier))
        data = NI.paginated_query(queries.phase_group_entrants, {'id': self.id})
        entrants = [Entrant.parse(entrant_data['entrant']) for entrant_data in data]
        return entrants

    def get_sets(self):
        Logger.info('Getting Sets for phase group: {0}:{1}'.format(self.id, self.display_identifier))
        data = NI.paginated_query(queries.phase_group_sets, {'id': self.id})
        sets = [GGSet.parse(set_data) for set_data in data]
        return sets

    def get_incomplete_sets(self):
        Logger.info('Getting Incomplete Sets for phase group: {0}:{1}'.format(self.id, self.display_identifier))
        sets = self.get_sets()
        incomplete_sets = []
        for gg_set in sets:
            if set.get_is_completed() is False:
                incomplete_sets.append(gg_set)
        return incomplete_sets

    def get_completed_sets(self):
        Logger.info('Getting Completed Sets for phase group: {0}:{1}'.format(self.id, self.display_identifier))
        sets = self.get_sets()
        incomplete_sets = []
        for gg_set in sets:
            if set.get_is_completed() is True:
                incomplete_sets.append(gg_set)
        return incomplete_sets

    # GETTERS
    def get_id(self):
        return self.id

    def get_display_identifier(self):
        return self.display_identifier

    def get_first_round_time(self):
        return self.first_round_time

    def get_state(self):
        return self.state

    def get_phase_id(self):
        return self.phase_id

    def get_wave_id(self):
        return self.wave_id

    def get_tiebreak_order(self):
        return self.tiebreak_order


from smashggpy.models.Entrant import Entrant
from smashggpy.models.Attendee import Attendee
from smashggpy.models.GGSet import GGSet
